@@ -81,6 +81,7 @@ static GtkActionEntry action_entries[] =
 {
   { "VNCListener",
     NULL,
+    /* Translators: "Reverse" here is an adjective, not a verb. */
     N_("_Reverse Connections..."),
     NULL,
     N_("Configure incoming VNC connections"),
@@ -274,12 +275,14 @@ impl_new_connection_from_file (VinagrePlugin *plugin,
 
   if (!g_key_file_has_group (file, "connection"))
     {
+      /* Translators: Do not translate "connection". It's the name of a group in the .vnc (.ini like) file. */
       *error_msg = g_strdup (_("The file is not a VNC one: Missing the group \"connection\"."));
       goto the_end;
     }
 
   if (!g_key_file_has_key (file, "connection", "host", NULL))
     {
+      /* Translators: Do not translate "host". It's the name of a key in the .vnc (.ini like) file. */
       *error_msg = g_strdup (_("The file is not a VNC one: Missing the key \"host\"."));
       goto the_end;
     }
@@ -318,6 +321,7 @@ impl_new_connection_from_file (VinagrePlugin *plugin,
       if (shared == 0 || shared == 1)
 	vinagre_vnc_connection_set_shared (VINAGRE_VNC_CONNECTION (conn), shared);
       else
+        /* Translators: 'shared' here is a VNC protocol specific flag. You can translate it, but I think it's better to let it untranslated */
 	g_message (_("Bad value for 'shared' flag: %d. It is supposed to be 0 or 1. Ignoring it."), shared);
     }
 
@@ -458,7 +462,12 @@ impl_get_connect_widget (VinagrePlugin *plugin, VinagreConnection *conn)
   gtk_widget_set_sensitive (ssh_host_entry, FALSE);
   g_object_set_data (G_OBJECT (box), "ssh_host", ssh_host_entry);
   /* Translators: This is the tooltip of the SSH tunneling entry */
-  gtk_widget_set_tooltip_text (ssh_host_entry, _("hostname or user@hostname"));
+  str = g_strdup_printf ("%s\n%s\n%s",
+			 _("hostname or user@hostname"),
+			 _("Supply an alternative port using colon"),
+			 _("For instance: joe@domain.com:5022"));
+  gtk_widget_set_tooltip_text (ssh_host_entry, str);
+  g_free (str);
   gtk_box_pack_start (GTK_BOX (box2), ssh_host_entry, FALSE, FALSE, 0);
 
   /* Translators: the whole sentence will be: Use host <hostname> as a SSH tunnel*/
