@@ -35,6 +35,7 @@
 #include "vinagre-utils.h"
 #include "vinagre-bookmarks.h"
 #include "vinagre-prefs.h"
+#include "vinagre-cache-prefs.h"
 #include "vinagre-plugins-engine.h"
 #include "vinagre-plugin.h"
 
@@ -83,6 +84,7 @@ protocol_combo_changed (GtkComboBox *combo, VinagreConnectDialog *dialog)
 
   if (!gtk_combo_box_get_active_iter (combo, &tree_iter))
     {
+      /* Translators: Please consider the word "from" instead of "in". I'll fix this phrase for 2.31 (or 3.0) */
       g_warning (_("Could not get the active protocol in the protocol list."));
       return;
     }
@@ -133,7 +135,7 @@ setup_protocol (VinagreConnectDialog *dialog)
 
   dialog->protocol_store = gtk_list_store_new (N_PROTOCOLS, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_OBJECT, G_TYPE_OBJECT);
   plugins = vinagre_plugin_engine_get_plugins_by_protocol (vinagre_plugins_engine_get_default ());
-  g_object_get (vinagre_prefs_get_default (), "last-protocol", &last_protocol, NULL);
+  last_protocol = vinagre_cache_prefs_get_string ("connection", "last-protocol", NULL);
 
   g_hash_table_iter_init (&hash_iter, plugins);
   selected = 0;
@@ -191,7 +193,7 @@ saved_history (void)
   gint len;
   GPtrArray *array;
 
-  array = g_ptr_array_new ();
+  array = g_ptr_array_new_with_free_func (g_free);
 
   filename = history_filename ();
   success = g_file_get_contents (filename,
@@ -203,7 +205,7 @@ saved_history (void)
     {
       history_from_file = g_strsplit (file_contents, "\n", 0);
       len = g_strv_length (history_from_file);
-      if (strlen (history_from_file[len-1]) == 0)
+      if (len > 0 && strlen (history_from_file[len-1]) == 0)
 	{
 	  g_free (history_from_file[len-1]);
 	  history_from_file[len-1] = NULL;
@@ -217,6 +219,7 @@ saved_history (void)
 
   g_free (filename);
   g_free (file_contents);
+  g_free (history_from_file);
   return array;
 }
 
@@ -320,9 +323,11 @@ static void
 vinagre_connect_find_button_cb (GtkButton            *button,
 				VinagreConnectDialog *dialog)
 {
-  GtkWidget   *d;
-  GtkTreeIter tree_iter;
-  gchar       *service;
+  GtkWidget     *d;
+  GtkTreeIter   tree_iter;
+  gchar         *service;
+  GtkWidget     *options = NULL;
+  VinagrePlugin *plugin = NULL;
 
   if (!gtk_combo_box_get_active_iter (GTK_COMBO_BOX (dialog->protocol_combo),
 				      &tree_iter))
@@ -333,9 +338,15 @@ vinagre_connect_find_button_cb (GtkButton            *button,
 
   gtk_tree_model_get (GTK_TREE_MODEL (dialog->protocol_store), &tree_iter,
 		      PROTOCOL_MDNS, &service,
+		      PROTOCOL_PLUGIN, &plugin,
+		      PROTOCOL_OPTIONS, &options,
 		      -1);
   if (!service)
-    return;
+    {
+      if (plugin)
+	g_object_unref (plugin);
+      return;
+    }
 
   d = aui_service_dialog_new (_("Choose a Remote Desktop"),
 				GTK_WINDOW(dialog->dialog),
@@ -365,10 +376,14 @@ vinagre_connect_find_button_cb (GtkButton            *button,
 			  tmp);
 
       g_free (tmp);
+      if (plugin && options)
+	vinagre_plugin_parse_mdns_dialog (plugin, options, d);
     }
 
   g_free (service);
   gtk_widget_destroy (d);
+  if (plugin)
+    g_object_unref (plugin);
 }
 #endif
 
@@ -378,7 +393,9 @@ VinagreConnection *vinagre_connect (VinagreWindow *window)
   gint                  result;
   VinagreConnectDialog  dialog;
 
-  dialog.xml = vinagre_utils_get_builder ();
+  dialog.xml = vinagre_utils_get_builder (NULL, NULL);
+  if (!dialog.xml)
+    return NULL;
 
   dialog.dialog = GTK_WIDGET (gtk_builder_get_object (dialog.xml, "connect_dialog"));
   gtk_window_set_transient_for (GTK_WINDOW (dialog.dialog), GTK_WINDOW (window));
@@ -393,6 +410,8 @@ VinagreConnection *vinagre_connect (VinagreWindow *window)
 
   setup_protocol (&dialog);
   setup_combo (&dialog);
+  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dialog.fullscreen_check),
+				vinagre_cache_prefs_get_boolean ("connection", "fullscreen", FALSE));
 
 #ifdef VINAGRE_ENABLE_AVAHI
   g_signal_connect (dialog.find_button,
@@ -435,8 +454,9 @@ VinagreConnection *vinagre_connect (VinagreWindow *window)
 			  PROTOCOL_PLUGIN, &plugin,
 		      -1);
 
-      g_object_set (vinagre_prefs_get_default (), "last-protocol", protocol, NULL);
+      vinagre_cache_prefs_set_string ("connection", "last-protocol", protocol);
       g_free (protocol);
+      vinagre_cache_prefs_set_boolean ("connection", "fullscreen", gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dialog.fullscreen_check)));
 
       conn = vinagre_plugin_new_connection (plugin);
       if (vinagre_connection_split_string (host,
