@@ -446,19 +446,25 @@ vinagre_about_dialog_handle_url (GtkAboutDialog *about,
 }
 
 void
-vinagre_utils_help_contents (GtkWindow *window)
+vinagre_utils_help_contents (GtkWindow *window, const gchar *section)
 {
   GError    *error;
   GdkScreen *screen;
+  gchar     *uri;
 
   screen = GTK_IS_WINDOW (window) ? gtk_window_get_screen (GTK_WINDOW (window)) : NULL;
   error = NULL;
+  if (section)
+    uri = g_strdup_printf ("ghelp:vinagre?%s", section);
+  else
+    uri = g_strdup ("ghelp:vinagre");
 
   gtk_show_uri (screen,
-		"ghelp:vinagre",
+		uri,
 		GDK_CURRENT_TIME,
 		&error);
 
+  g_free (uri);
   if (error != NULL) 
     {
       vinagre_utils_show_error (NULL, error->message, GTK_IS_WINDOW (window) ? window : NULL);
@@ -601,11 +607,148 @@ vinagre_utils_ask_question (GtkWindow  *parent,
   return TRUE;
 }
 
-#include "vinagre-ssh.h"
+typedef struct {
+  GtkWidget *uname, *pw, *button;
+} ControlOKButton;
+
 static void
-shit (void)
+control_ok_button (GtkEditable *entry, ControlOKButton *data)
 {
-  vinagre_ssh_connect (NULL, -1, NULL, NULL, NULL, NULL, NULL);
+  gboolean enabled = TRUE;
+
+  if (GTK_WIDGET_VISIBLE (data->uname))
+    enabled = enabled && gtk_entry_get_text_length (GTK_ENTRY (data->uname)) > 0;
+
+  if (GTK_WIDGET_VISIBLE (data->pw))
+    enabled = enabled && gtk_entry_get_text_length (GTK_ENTRY (data->pw)) > 0;
+
+  gtk_widget_set_sensitive (data->button, enabled);
+}
+
+gboolean
+vinagre_utils_ask_credential (GtkWindow *parent,
+			      gchar *kind,
+			      gchar *host,
+			      gboolean need_username,
+			      gboolean need_password,
+			      gint password_limit,
+			      gchar **username,
+			      gchar **password,
+			      gboolean *save_in_keyring)
+{
+  GtkBuilder      *xml;
+  GtkWidget       *password_dialog, *save_credential_check;
+  GtkWidget       *password_label, *username_label, *image;
+  int             result;
+  ControlOKButton control;
+
+  xml = vinagre_utils_get_builder (NULL, NULL);
+
+  password_dialog = GTK_WIDGET (gtk_builder_get_object (xml, "auth_required_dialog"));
+  if (parent)
+    gtk_window_set_transient_for (GTK_WINDOW (password_dialog), parent);
+
+  if (kind)
+    {
+       /* Translators: %s is a protocol, like VNC or SSH */
+       gchar *str = g_strdup_printf (_("%s authentication is required"), kind);
+       GtkWidget *auth_label = GTK_WIDGET (gtk_builder_get_object (xml, "auth_required_label"));
+       gtk_label_set_label (GTK_LABEL (auth_label), str);
+       g_free (str);
+    }
+
+  if (host)
+    {
+       GtkWidget *host_label = GTK_WIDGET (gtk_builder_get_object (xml, "host_label"));
+       gtk_label_set_label (GTK_LABEL (host_label), host);
+    }
+
+  control.uname  = GTK_WIDGET (gtk_builder_get_object (xml, "username_entry"));
+  control.pw     = GTK_WIDGET (gtk_builder_get_object (xml, "password_entry"));
+  control.button = GTK_WIDGET (gtk_builder_get_object (xml, "ok_button"));
+  password_label = GTK_WIDGET (gtk_builder_get_object (xml, "password_label"));
+  username_label = GTK_WIDGET (gtk_builder_get_object (xml, "username_label"));
+  save_credential_check = GTK_WIDGET (gtk_builder_get_object (xml, "save_credential_check"));
+
+  image = gtk_image_new_from_stock (GTK_STOCK_DIALOG_AUTHENTICATION, GTK_ICON_SIZE_BUTTON);
+  gtk_button_set_image (GTK_BUTTON (control.button), image);
+
+  g_signal_connect (control.uname, "changed", G_CALLBACK (control_ok_button), &control);
+  g_signal_connect (control.pw, "changed", G_CALLBACK (control_ok_button), &control);
+
+  if (need_username)
+    {
+      if (username && *username)
+        gtk_entry_set_text (GTK_ENTRY (control.uname), *username);
+    }
+  else
+    {
+      gtk_widget_hide (username_label);
+      gtk_widget_hide (control.uname);
+    }
+
+  if (need_password)
+    {
+      gtk_entry_set_max_length (GTK_ENTRY (control.pw), password_limit);
+      if (password && *password)
+        gtk_entry_set_text (GTK_ENTRY (control.pw), *password);
+    }
+  else
+    {
+      gtk_widget_hide (password_label);
+      gtk_widget_hide (control.pw);
+    }
+
+  result = gtk_dialog_run (GTK_DIALOG (password_dialog));
+  if (result == -5)
+    {
+      if (username)
+        {
+          g_free (*username);
+          if (gtk_entry_get_text_length (GTK_ENTRY (control.uname)) > 0)
+	    *username = g_strdup (gtk_entry_get_text (GTK_ENTRY (control.uname)));
+          else
+	    *username = NULL;
+	}
+
+      if (password)
+        {
+          g_free (*password);
+          if (gtk_entry_get_text_length (GTK_ENTRY (control.pw)) > 0)
+	    *password = g_strdup (gtk_entry_get_text (GTK_ENTRY (control.pw)));
+          else
+	    *password = NULL;
+	}
+
+      if (save_in_keyring)
+        *save_in_keyring = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (save_credential_check));
+    }
+
+  gtk_widget_destroy (GTK_WIDGET (password_dialog));
+  g_object_unref (xml);
+
+  return result == -5;
+}
+
+gboolean
+vinagre_utils_create_dir (const gchar *filename, GError **error)
+{
+  GFile    *file, *parent;
+  gboolean result;
+  gchar    *path;
+
+  file   = g_file_new_for_path (filename);
+  parent = g_file_get_parent (file);
+  path   = g_file_get_path (parent);
+  result = TRUE;
+
+  if (!g_file_test (path, G_FILE_TEST_EXISTS))
+    result = g_file_make_directory_with_parents (parent, NULL, error);
+
+  g_object_unref (file);
+  g_object_unref (parent);
+  g_free (path);
+  return result;
 }
 
 /* vim: set ts=8: */
