@@ -22,6 +22,7 @@
 #include <config.h>
 #endif
 
+#include <errno.h>
 #include <time.h>
 #include <sys/types.h>
 #include <string.h>
@@ -33,6 +34,7 @@
 #include "vinagre-notebook.h"
 #include "vinagre-fav.h"
 #include "vinagre-prefs.h"
+#include "vinagre-cache-prefs.h"
 #include "vinagre-utils.h"
 #include "vinagre-bookmarks.h"
 #include "vinagre-ui.h"
@@ -139,9 +141,7 @@ vinagre_window_state_event_cb (GtkWidget *widget,
   VinagreWindow *window = VINAGRE_WINDOW (widget);
 
   window->priv->window_state = event->new_window_state;
-  g_object_set (vinagre_prefs_get_default (),
-		"window-state", window->priv->window_state,
-		NULL);
+  vinagre_cache_prefs_set_integer ("window", "window-state", window->priv->window_state);
 
   if ((event->changed_mask & GDK_WINDOW_STATE_FULLSCREEN) == 0)
     return FALSE;
@@ -165,13 +165,14 @@ vinagre_window_configure_event (GtkWidget         *widget,
 {
   VinagreWindow *window = VINAGRE_WINDOW (widget);
 
-  window->priv->width  = event->width;
-  window->priv->height = event->height;
+  if ((vinagre_cache_prefs_get_integer ("window", "window-state", 0) & GDK_WINDOW_STATE_MAXIMIZED) == 0)
+    {
+      window->priv->width  = event->width;
+      window->priv->height = event->height;
 
-  g_object_set (vinagre_prefs_get_default (),
-		"window-width", window->priv->width,
-		"window-height", window->priv->height,
-		NULL);
+      vinagre_cache_prefs_set_integer ("window", "window-width", window->priv->width);
+      vinagre_cache_prefs_set_integer ("window", "window-height", window->priv->height);
+    }
 
   return GTK_WIDGET_CLASS (vinagre_window_parent_class)->configure_event (widget, event);
 }
@@ -475,9 +476,7 @@ fav_panel_size_allocate (GtkWidget     *widget,
 {
   window->priv->side_panel_size = allocation->width;
   if (window->priv->side_panel_size > 0)
-    g_object_set (vinagre_prefs_get_default (),
-		  "side-panel-size", window->priv->side_panel_size,
-		  NULL);
+    vinagre_cache_prefs_set_integer ("window", "side-panel-size", window->priv->side_panel_size);
 }
 
 static void
@@ -625,9 +624,7 @@ create_side_panel (VinagreWindow *window)
 		   FALSE, 
 		   FALSE);
 
-  g_object_get (vinagre_prefs_get_default (),
-		"side-panel-size", &(window->priv->side_panel_size),
-		NULL);
+  window->priv->side_panel_size = vinagre_cache_prefs_get_integer ("window", "side-panel-size", 200);
   gtk_paned_set_position (GTK_PANED (window->priv->hpaned), window->priv->side_panel_size);
 
   g_signal_connect (window->priv->fav_panel,
@@ -639,42 +636,35 @@ create_side_panel (VinagreWindow *window)
 static void
 init_widgets_visibility (VinagreWindow *window)
 {
-  GdkWindowState state;
-  gint w, h;
   GtkAction *action;
   gboolean visible;
-  VinagrePrefs *prefs = vinagre_prefs_get_default ();
 
   /* side panel visibility */
   action = gtk_action_group_get_action (window->priv->always_sensitive_action_group,
 					"ViewSidePanel");
-  g_object_get (prefs, "side-panel-visible", &visible, NULL);
+  visible = vinagre_cache_prefs_get_boolean ("window", "side-panel-visible", TRUE);
   if (gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action)) != visible)
     gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action), visible);
 
   /* toolbar visibility */
   action = gtk_action_group_get_action (window->priv->always_sensitive_action_group,
 					"ViewToolbar");
-  g_object_get (prefs, "toolbar-visible", &visible, NULL);
+  visible = vinagre_cache_prefs_get_boolean ("window", "toolbar-visible", TRUE);
   if (gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action)) != visible)
     gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action), visible);
 
   /* statusbar visibility */
   action = gtk_action_group_get_action (window->priv->always_sensitive_action_group,
 					"ViewStatusbar");
-  g_object_get (prefs, "statusbar-visible", &visible, NULL);
+  visible = vinagre_cache_prefs_get_boolean ("window", "statusbar-visible", TRUE);
   if (gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action)) != visible)
     gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action), visible);
 
-  g_object_get (prefs,
-		"window-state", &state,
-		"window-width", &w,
-		"window-height", &h,
-		NULL);
+  gtk_window_set_default_size (GTK_WINDOW (window),
+			       vinagre_cache_prefs_get_integer ("window", "window-width", 650),
+			       vinagre_cache_prefs_get_integer ("window", "window-height", 500));
 
-  gtk_window_set_default_size (GTK_WINDOW (window), w, h);
-
-  if ((state & GDK_WINDOW_STATE_MAXIMIZED) != 0)
+  if ((vinagre_cache_prefs_get_integer ("window", "window-state", 0) & GDK_WINDOW_STATE_MAXIMIZED) != 0)
     gtk_window_maximize (GTK_WINDOW (window));
   else
     gtk_window_unmaximize (GTK_WINDOW (window));
@@ -711,6 +701,48 @@ create_notebook (VinagreWindow *window)
   gtk_widget_show (GTK_WIDGET (window->priv->notebook));
 }
 
+static gboolean
+vinagre_window_check_first_run (VinagreWindow *window)
+{
+  gchar *filename;
+
+  filename = g_build_filename (g_get_user_data_dir (),
+			       "vinagre",
+			       "first_run",
+			       NULL);
+
+  if (!g_file_test (filename, G_FILE_TEST_EXISTS))
+    {
+      GError *error = NULL;
+      GtkWidget *dialog = gtk_message_dialog_new (GTK_WINDOW (window),
+						  GTK_DIALOG_DESTROY_WITH_PARENT | GTK_DIALOG_MODAL,
+						  GTK_MESSAGE_INFO,
+						  GTK_BUTTONS_CLOSE,
+						  _("About menu accelerators and keyboard shortcuts"));
+      gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
+						_("Vinagre comes with menu accelerators and keyboard shortcuts disabled by default. The reason is to avoid the keys to be intercepted by the program, and allow them to be sent to the remote machine.\n\nYou can change this behavior through the preferences dialog. For more information, check the documentation.\n\nThis message will appear only once."));     
+      gtk_dialog_run (GTK_DIALOG (dialog));
+      gtk_widget_destroy (dialog);
+
+      if (vinagre_utils_create_dir (filename, &error))
+        {
+          int fd = g_creat (filename, 0644);
+          if (fd < 0)
+            g_warning (_("Error while creating the file %s: %s"), filename, strerror (errno));
+          else
+            close (fd);
+        }
+      else
+        {
+          g_warning (_("Error while creating the file %s: %s"), filename, error ? error->message: _("Unknown error"));
+          g_clear_error (&error);
+        }
+    }
+
+  g_free (filename);
+  return FALSE;
+}
+
 static void
 vinagre_window_init (VinagreWindow *window)
 {
@@ -768,6 +800,8 @@ vinagre_window_init (VinagreWindow *window)
   vinagre_plugins_engine_activate_plugins (vinagre_plugins_engine_get_default (),
 					   window);
 
+
+  g_idle_add ((GSourceFunc) vinagre_window_check_first_run, window);
 }
 
 VinagreNotebook *
